@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24880 $
+ * $Revision: 29485 $
  * $Author: rjongbloed $
- * $Date: 2010-11-18 01:23:46 -0600 (Thu, 18 Nov 2010) $
+ * $Date: 2013-04-10 20:51:52 -0500 (Wed, 10 Apr 2013) $
  */
 
 #include <ptlib.h>
@@ -163,8 +163,11 @@ PBoolean H323_RTP_UDP::ExtractTransport(const H245_TransportAddress & pdu,
 
   PIPSocket::Address ip;
   WORD port = 0;
-  if (transAddr.GetIpAndPort(ip, port) && ip.IsValid() && port != 0)
-    return rtp.SetRemoteSocketInfo(ip, port, isDataPort);
+  if (transAddr.GetIpAndPort(ip, port) && ip.IsValid() && port != 0) {
+    if (isDataPort || rtp.GetRemoteDataPort() == 0)
+      return rtp.SetRemoteSocketInfo(ip, port, isDataPort);
+    return true;
+  }
 
   PTRACE(1, "RTP_UDP\tIllegal IP address/port in transport address.");
   return PFalse;
@@ -195,21 +198,26 @@ PBoolean H323_RTP_UDP::OnReceivedPDU(H323_RTPChannel & channel,
     ok = PTrue;
   }
 
-  if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_dynamicRTPPayloadType))
-    channel.SetDynamicRTPPayloadType(param.m_dynamicRTPPayloadType);
+  OpalMediaFormat mediaFormat = channel.GetMediaStream()->GetMediaFormat();
 
+  // Override default payload type if indicated by remote
+  RTP_DataFrame::PayloadTypes rtpPayloadType = mediaFormat.GetPayloadType();
+  if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_dynamicRTPPayloadType))
+    rtpPayloadType = (RTP_DataFrame::PayloadTypes)param.m_dynamicRTPPayloadType.GetValue();
+
+  // Update actual media packetization if present
   PString mediaPacketization;
   if (param.HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaPacketization) &&
       param.m_mediaPacketization.GetTag() == H245_H2250LogicalChannelParameters_mediaPacketization::e_rtpPayloadType)
     mediaPacketization = H323GetRTPPacketization(param.m_mediaPacketization);
 
-  OpalMediaFormat mediaFormat = channel.GetMediaStream()->GetMediaFormat();
-  if (mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption()) != mediaPacketization ||
-      mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption())  != mediaPacketization) {
-    mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), mediaPacketization);
-    mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(), mediaPacketization);
-    channel.GetMediaStream()->UpdateMediaFormat(mediaFormat);
-  }
+  // Hack for H.263 compatibility, default to RFC2190 if not told otherwise
+  if (mediaPacketization.IsEmpty() && mediaFormat == OPAL_H263)
+    mediaPacketization = "RFC2190";
+
+  mediaFormat.SetPayloadType(rtpPayloadType);
+  mediaFormat.SetMediaPacketizations(mediaPacketization);
+  channel.GetMediaStream()->UpdateMediaFormat(mediaFormat);
 
   if (ok)
     return PTrue;
@@ -251,8 +259,11 @@ PBoolean H323_RTP_UDP::OnReceivedAckPDU(H323_RTPChannel & channel,
   if (!ExtractTransport(param.m_mediaChannel, PTrue, errorCode))
     return PFalse;
 
-  if (param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_dynamicRTPPayloadType))
-    channel.SetDynamicRTPPayloadType(param.m_dynamicRTPPayloadType);
+  if (param.HasOptionalField(H245_H2250LogicalChannelAckParameters::e_dynamicRTPPayloadType)) {
+    OpalMediaFormat mediaFormat = channel.GetMediaStream()->GetMediaFormat();
+    mediaFormat.SetPayloadType((RTP_DataFrame::PayloadTypes)param.m_dynamicRTPPayloadType.GetValue());
+    channel.GetMediaStream()->UpdateMediaFormat(mediaFormat);
+  }
 
   return PTrue;
 }
