@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 23295 $
- * $Author: rjongbloed $
- * $Date: 2009-08-28 01:38:49 -0500 (Fri, 28 Aug 2009) $
+ * $Revision: 29598 $
+ * $Author: ededu $
+ * $Date: 2013-04-29 12:22:45 -0500 (Mon, 29 Apr 2013) $
  */
 
 #include <ptlib.h>
@@ -101,12 +101,12 @@ OpalTransportAddress::OpalTransportAddress(const PIPSocket::Address & addr, WORD
 }
 
 
-PString OpalTransportAddress::GetHostName() const
+PString OpalTransportAddress::GetHostName(bool includeService) const
 {
   if (transport == NULL)
     return *this;
 
-  return transport->GetHostName(*this);
+  return transport->GetHostName(*this, includeService);
 }
 
 
@@ -132,8 +132,8 @@ PBoolean OpalTransportAddress::IsCompatible(const OpalTransportAddress & address
   if (IsEmpty() || address.IsEmpty())
     return PTrue;
 
-  PCaselessString myPrefix = GetProto();
-  PCaselessString theirPrefix = address.GetProto();
+  PCaselessString myPrefix = GetProto(true);
+  PCaselessString theirPrefix = address.GetProto(true);
   return myPrefix == theirPrefix ||
         (myPrefix    == IpPrefix && (theirPrefix == TcpPrefix || theirPrefix == UdpPrefix 
 #if OPAL_PTLIB_SSL
@@ -271,7 +271,7 @@ void OpalTransportAddressArray::AppendStringCollection(const PCollection & coll)
 
 /////////////////////////////////////////////////////////////////
 
-PString OpalInternalTransport::GetHostName(const OpalTransportAddress & address) const
+PString OpalInternalTransport::GetHostName(const OpalTransportAddress & address, bool) const
 {
   // skip transport identifier
   PINDEX pos = address.Find('$');
@@ -366,20 +366,23 @@ static PBoolean SplitAddress(const PString & addr, PString & host, PString & dev
 }
 
 
-PString OpalInternalIPTransport::GetHostName(const OpalTransportAddress & address) const
+PString OpalInternalIPTransport::GetHostName(const OpalTransportAddress & address, bool includeService) const
 {
   PString host, device, service;
   if (!SplitAddress(address, host, device, service))
     return address;
 
-  if (!device.IsEmpty())
-    return host+device;
+  PString hostname = host+device;
 
-  PIPSocket::Address ip;
-  if (ip.FromString(host))
-    return ip.AsString(true);
+  if (device.IsEmpty()) {
+    PIPSocket::Address ip;
+    if (ip.FromString(host))
+      hostname = ip.AsString(true);
+  }
 
-  return host;
+  if (includeService)
+    hostname += ':' + service;
+  return hostname;
 }
 
 
@@ -411,15 +414,21 @@ PBoolean OpalInternalIPTransport::GetIpAndPort(const OpalTransportAddress & addr
     }
   }
 
-  if (host[0] == '*' || host == "0.0.0.0"
-#ifdef P_HAS_IPV6
-	  || host == ":" || host == "::"  || host == "[::]"
-#endif
-	  ) {
+  if (host[0] == '*') {
     ip = PIPSocket::GetDefaultIpAny();
     return PTrue;
   }
 
+  if (host == "0.0.0.0") {
+    ip = PIPSocket::Address::GetAny(4);
+    return true;
+  }
+
+  if (host == "::" || host == "[::]") {
+    ip = PIPSocket::Address::GetAny(6);
+    return true;
+  }
+
   if (device.IsEmpty()) {
     if (PIPSocket::GetHostAddress(host, ip))
       return true;
@@ -722,8 +731,9 @@ OpalListenerUDP::OpalListenerUDP(OpalEndPoint & endpoint,
 OpalListenerUDP::OpalListenerUDP(OpalEndPoint & endpoint,
                                  const OpalTransportAddress & binding,
                                  OpalTransportAddress::BindOptions option)
-  : OpalListenerIP(endpoint, binding, option),
-    listenerBundle(PMonitoredSockets::Create(binding.GetHostName(), !exclusiveListener, endpoint.GetManager().GetNatMethod()))
+  : OpalListenerIP(endpoint, binding, option)
+  , listenerBundle(PMonitoredSockets::Create(binding.GetHostName(), !exclusiveListener, endpoint.GetManager().GetNatMethod()))
+  , m_bufferSize(32768)
 {
 }
 
@@ -773,21 +783,31 @@ OpalTransport * OpalListenerUDP::Accept(const PTimeInterval & timeout)
   WORD remotePort;
   PString iface;
   PINDEX readCount;
-  static const PINDEX SixtyFourK = 0x10000;
-  switch (listenerBundle->ReadFromBundle(pdu.GetPointer(SixtyFourK), SixtyFourK, remoteAddr, remotePort, iface, readCount, timeout)) {
+  bool preReadOK;
+  switch (listenerBundle->ReadFromBundle(pdu.GetPointer(m_bufferSize), m_bufferSize, remoteAddr, remotePort, iface, readCount, timeout)) {
     case PChannel::NoError :
       pdu.SetSize(readCount);
-      return new OpalTransportUDP(endpoint, pdu, listenerBundle, iface, remoteAddr, remotePort);
+      preReadOK = true;
+      break;
+
+    case PChannel::BufferTooSmall :
+      preReadOK = false;
+      break;
 
     case PChannel::Interrupted :
       PTRACE(4, "Listen\tInterfaces changed");
-      break;
+      return NULL;
 
     default :
       PTRACE(1, "Listen\tUDP read error.");
+      return NULL;
   }
 
-  return NULL;
+  OpalTransportUDP * transport = new OpalTransportUDP(endpoint, listenerBundle, iface);
+  transport->m_preReadPacket = pdu;
+  transport->m_preReadOK = preReadOK;
+  transport->SetRemoteAddress(OpalTransportAddress(remoteAddr, remotePort, "udp"));
+  return transport;
 }
 
 
@@ -798,14 +818,11 @@ OpalTransport * OpalListenerUDP::CreateTransport(const OpalTransportAddress & lo
     return NULL;
 
   PIPSocket::Address addr;
-  if (remoteAddress.GetIpAddress(addr) && addr.IsLoopback())
-    return new OpalTransportUDP(endpoint, addr);
-
   PString iface;
   if (localAddress.GetIpAddress(addr))
     iface = addr.AsString(true);
 
-  return new OpalTransportUDP(endpoint, PBYTEArray(), listenerBundle, iface, PIPSocket::GetDefaultIpAny(), 0);
+  return new OpalTransportUDP(endpoint, listenerBundle, iface);
 }
 
 
@@ -1067,7 +1084,7 @@ PBoolean OpalTransportTCP::Connect()
     PTRACE(4, "OpalTCP\tConnecting to "
            << remoteAddress.AsString(true) << ':' << remotePort
            << " (local port=" << localPort << ')');
-    if (socket->Connect(localPort, remoteAddress))
+    if (socket->Connect(localAddress, localPort, remoteAddress))
       break;
 
     int errnum = socket->GetErrorNumber();
@@ -1207,6 +1224,8 @@ OpalTransportUDP::OpalTransportUDP(OpalEndPoint & ep,
                                    bool preOpen)
   : OpalTransportIP(ep, binding, localPort)
   , manager(ep.GetManager())
+  , m_bufferSize(8192)
+  , m_preReadOK(false)
 {
   PMonitoredSockets * sockets = PMonitoredSockets::Create(binding.AsString(), reuseAddr, manager.GetNatMethod());
   if (preOpen)
@@ -1216,20 +1235,14 @@ OpalTransportUDP::OpalTransportUDP(OpalEndPoint & ep,
 
 
 OpalTransportUDP::OpalTransportUDP(OpalEndPoint & ep,
-                                   const PBYTEArray & packet,
                                    const PMonitoredSocketsPtr & listener,
-                                   const PString & iface,
-                                   PIPSocket::Address remAddr,
-                                   WORD remPort)
+                                   const PString & iface)
   : OpalTransportIP(ep, PIPSocket::GetDefaultIpAny(), 0)
   , manager(ep.GetManager())
-  , preReadPacket(packet)
+  , m_bufferSize(8192)
+  , m_preReadOK(true)
 {
-  remoteAddress = remAddr;
-  remotePort = remPort;
-
   PMonitoredSocketChannel * socket = new PMonitoredSocketChannel(listener, PTrue);
-  socket->SetRemote(remAddr, remPort);
   socket->SetInterface(iface);
   socket->GetLocal(localAddress, localPort, !manager.IsLocalAddress(remoteAddress));
   Open(socket);
@@ -1323,7 +1336,9 @@ OpalTransportAddress OpalTransportUDP::GetLocalAddress(bool allowNAT) const
   PMonitoredSocketChannel * socket = (PMonitoredSocketChannel *)readChannel;
   if (socket != NULL) {
     OpalTransportUDP * thisWritable = const_cast<OpalTransportUDP *>(this);
-    socket->GetLocal(thisWritable->localAddress, thisWritable->localPort, allowNAT && !manager.IsLocalAddress(remoteAddress));
+    if (!socket->GetLocal(thisWritable->localAddress, thisWritable->localPort,
+                          allowNAT && !manager.IsLocalAddress(remoteAddress)))
+      return OpalTransportAddress();
   }
 
   return OpalTransportIP::GetLocalAddress(allowNAT);
@@ -1405,28 +1420,28 @@ PString OpalTransportUDP::GetLastReceivedInterface() const
 
 PBoolean OpalTransportUDP::Read(void * buffer, PINDEX length)
 {
-  if (preReadPacket.IsEmpty())
+  if (m_preReadPacket.IsEmpty())
     return OpalTransportIP::Read(buffer, length);
 
-  lastReadCount = PMIN(length, preReadPacket.GetSize());
-  memcpy(buffer, preReadPacket, lastReadCount);
-  preReadPacket.SetSize(0);
+  lastReadCount = PMIN(length, m_preReadPacket.GetSize());
+  memcpy(buffer, m_preReadPacket, lastReadCount);
+  m_preReadPacket.SetSize(0);
 
-  return PTrue;
+  return m_preReadOK;
 }
 
 
 PBoolean OpalTransportUDP::ReadPDU(PBYTEArray & packet)
 {
-  if (preReadPacket.GetSize() > 0) {
-    packet = preReadPacket;
-    preReadPacket.SetSize(0);
-    return PTrue;
+  if (m_preReadPacket.GetSize() > 0) {
+    packet = m_preReadPacket;
+    m_preReadPacket.SetSize(0);
+    return m_preReadOK;
   }
 
-  if (!Read(packet.GetPointer(10000), 10000)) {
+  if (!Read(packet.GetPointer(m_bufferSize), m_bufferSize)) {
     packet.SetSize(0);
-    return PFalse;
+    return false;
   }
 
   packet.SetSize(GetLastReadCount());
@@ -1453,10 +1468,16 @@ PBoolean OpalTransportUDP::WriteConnect(WriteConnectCallback function, void * us
 
   PBoolean ok = PFalse;
   for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
-    PTRACE(4, "OpalUDP\tWriting to interface " << i << " - \"" << interfaces[i] << '"');
-    socket->SetInterface(interfaces[i]);
-    if (function(*this, userData))
-      ok = PTrue;
+    PIPSocket::Address ifip(interfaces[i]);
+    if (ifip.GetVersion() != remoteAddress.GetVersion())
+      PTRACE(4, "OpalUDP\tSkipping incompatible interface " << i << " - \"" << interfaces[i] << '"');
+    else {
+      PTRACE(4, "OpalUDP\tWriting to interface " << i << " - \"" << interfaces[i] << '"');
+      socket->SetInterface(interfaces[i]);
+      // Make sure is compatible address
+      if (function(*this, userData))
+        ok = PTrue;
+    }
   }
 
   return ok;
@@ -1554,7 +1575,7 @@ PBoolean OpalTransportTCPS::Connect()
     PTRACE(4, "OpalTCPS\tConnecting to "
            << remoteAddress.AsString(true) << ':' << remotePort
            << " (local port=" << localPort << ')');
-    if (socket->Connect(localPort, remoteAddress))
+    if (socket->Connect(localAddress, localPort, remoteAddress))
       break;
 
     int errnum = socket->GetErrorNumber();
